@@ -22,7 +22,6 @@
 
 #include "miscadmin.h"
 #include "nodes/makefuncs.h"
-#include "parser/keywords.h"
 #include "parser/parse_type.h"
 #include "executor/spi.h"
 #include "commands/trigger.h"
@@ -473,7 +472,13 @@ versionFunc(logTrigger) (PG_FUNCTION_ARGS)
 	/*
 	 * Save the current datestyle setting and switch to ISO (if not already)
 	 */
+#ifdef GETCONFIGOPTIONBYNAME_2
 	olddatestyle = GetConfigOptionByName("DateStyle", NULL);
+#elif defined (GETCONFIGOPTIONBYNAME_3)
+	olddatestyle = GetConfigOptionByName("DateStyle", NULL, false);
+#else
+#error "GETCONFIGOPTIONBYNAME_2 or GETCONFIGOPTIONBYNAME_3 must be defined"
+#endif
 	if (!strstr(olddatestyle, "ISO"))
 	{
 #ifdef SETCONFIGOPTION_6
